@@ -1152,19 +1152,6 @@ gail_entry_get_character_at_offset (AtkText *text,
   return unichar;
 }
 
-static void
-notify_delete (gpointer data)
-{
-  GailEntry *entry;
-
-  GDK_THREADS_ENTER ();
-
-  entry = GAIL_ENTRY (data);
-  gail_entry_notify_delete (entry);
-
-  GDK_THREADS_LEAVE ();
-}
-
 static void
 gail_entry_notify_delete (GailEntry *entry)
 {
@@ -1203,7 +1190,7 @@ _gail_entry_delete_text_cb (GtkEntry *entry,
       gail_entry->position_delete = arg1;
       gail_entry->length_delete = arg2 - arg1;
     }
-  notify_delete (gail_entry);
+  gail_entry_notify_delete (gail_entry);
 }
 
 static void
