@@ -168,10 +168,13 @@ gail_statusbar_new (GtkWidget *widget)
 static G_CONST_RETURN gchar*
 gail_statusbar_get_name (AtkObject *obj)
 {
+  G_CONST_RETURN gchar* name;
+
   g_return_val_if_fail (GAIL_IS_STATUSBAR (obj), NULL);
 
-  if (ATK_OBJECT(obj)->name != NULL)
-    return ATK_OBJECT(obj)->name;
+  name = ATK_OBJECT_CLASS (parent_class)->get_name (obj);
+  if (name != NULL)
+    return name;
   else
     {
       /*
@@ -197,19 +200,54 @@ gail_statusbar_get_name (AtkObject *obj)
 }
 
 static gint
-gail_statusbar_get_n_children (AtkObject          *obj)
+gail_statusbar_get_n_children (AtkObject *obj)
 {
-  /*
-   * We report a GailStatusbar as having no children
-   */
-  return 0;
+  GtkWidget *widget;
+  GList *children;
+  gint count = 0;
+
+  widget = GTK_ACCESSIBLE (obj)->widget;
+  if (widget == NULL)
+    return 0;
+
+  children = gtk_container_get_children (GTK_CONTAINER (widget));
+  if (children != NULL)
+    {
+      count = g_list_length (children);
+      g_list_free (children);
+    }
+
+  return count;
 }
 
 static AtkObject*
-gail_statusbar_ref_child (AtkObject          *obj,
-                          gint               i)
+gail_statusbar_ref_child (AtkObject *obj,
+                          gint      i)
 {
-  return NULL;
+  GList *children, *tmp_list;
+  AtkObject  *accessible;
+  GtkWidget *widget;
+
+  g_return_val_if_fail ((i >= 0), NULL);
+  widget = GTK_ACCESSIBLE (obj)->widget;
+  if (widget == NULL)
+    return NULL;
+
+  children = gtk_container_get_children (GTK_CONTAINER (widget));
+  if (children == NULL)
+    return NULL;
+
+  tmp_list = g_list_nth (children, i);
+  if (!tmp_list)
+    {
+      g_list_free (children);
+      return NULL;
+    }
+  accessible = gtk_widget_get_accessible (GTK_WIDGET (tmp_list->data));
+
+  g_list_free (children);
+  g_object_ref (accessible);
+  return accessible;
 }
 
 static void
