@@ -17,8 +17,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <sys/types.h>
+#include <unistd.h>
 #include <string.h>
 #include <stdlib.h>
+#include <glib-object.h>
+#include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include "gailtextview.h"
 #include <libgail-util/gailmisc.h>
@@ -124,6 +128,16 @@ static void       gail_text_view_paste_text            (AtkEditableText  *text,
 static void       gail_text_view_paste_received        (GtkClipboard     *clipboard,
                                                         const gchar      *text,
                                                         gpointer         data);
+/* AtkStreamableContent */
+static void       atk_streamable_content_interface_init    (AtkStreamableContentIface *iface);
+static gint       gail_streamable_content_get_n_mime_types (AtkStreamableContent *streamable);
+static G_CONST_RETURN gchar* gail_streamable_content_get_mime_type (AtkStreamableContent *streamable,
+								    gint i);
+static GIOChannel* gail_streamable_content_get_stream       (AtkStreamableContent *streamable,
+							     const gchar *mime_type);
+/* getURI requires atk-1.12.0 or later
+static void       gail_streamable_content_get_uri          (AtkStreamableContent *streamable);
+*/
 
 /* Callbacks */
 
@@ -199,12 +213,21 @@ gail_text_view_get_type (void)
         NULL
       };
 
+      static const GInterfaceInfo atk_streamable_content_info =
+      {
+        (GInterfaceInitFunc) atk_streamable_content_interface_init,
+        (GInterfaceFinalizeFunc) NULL,
+        NULL
+      };
+
       type = g_type_register_static (GAIL_TYPE_CONTAINER,
                                      "GailTextView", &tinfo, 0);
       g_type_add_interface_static (type, ATK_TYPE_EDITABLE_TEXT,
                                    &atk_editable_text_info);
       g_type_add_interface_static (type, ATK_TYPE_TEXT,
                                    &atk_text_info);
+      g_type_add_interface_static (type, ATK_TYPE_STREAMABLE_CONTENT,
+                                   &atk_streamable_content_info);
     }
 
   return type;
@@ -240,19 +263,15 @@ gail_text_view_init (GailTextView      *text_view)
 }
 
 static void
-gail_text_view_real_initialize (AtkObject *obj,
-                                gpointer  data)
+setup_buffer (GtkTextView  *view, 
+              GailTextView *gail_view)
 {
-  GtkTextView *view;
   GtkTextBuffer *buffer;
-  GailTextView *gail_view;
 
-  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
-
-  view = GTK_TEXT_VIEW (data);
   buffer = view->buffer;
+  if (buffer == NULL)
+    return;
 
-  gail_view = GAIL_TEXT_VIEW (obj);
   gail_view->textutil = gail_text_util_new ();
   gail_text_util_buffer_setup (gail_view->textutil, buffer);
 
@@ -266,6 +285,22 @@ gail_text_view_real_initialize (AtkObject *obj,
   g_signal_connect_data (buffer, "changed",
      (GCallback) _gail_text_view_changed_cb, view, NULL, 0);
 
+}
+
+static void
+gail_text_view_real_initialize (AtkObject *obj,
+                                gpointer  data)
+{
+  GtkTextView *view;
+  GailTextView *gail_view;
+
+  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
+
+  view = GTK_TEXT_VIEW (data);
+
+  gail_view = GAIL_TEXT_VIEW (obj);
+  setup_buffer (view, gail_view);
+
   obj->role = ATK_ROLE_TEXT;
 
 }
@@ -312,6 +347,13 @@ gail_text_view_real_notify_gtk (GObject             *obj,
       atk_object_notify_state_change (atk_obj, ATK_STATE_EDITABLE,
                                       editable);
     }
+  else if (!strcmp (pspec->name, "buffer"))
+    {
+      AtkObject *atk_obj;
+
+      atk_obj = gtk_widget_get_accessible (GTK_WIDGET (obj));
+      setup_buffer (GTK_TEXT_VIEW (obj), GAIL_TEXT_VIEW (atk_obj));
+    }
   else
     parent_class->notify_gtk (obj, pspec);
 }
@@ -827,7 +869,7 @@ gail_text_view_get_selection (AtkText *text,
   * selection_num is 0.
   */
   if (selection_num != 0)
-     return 0;
+     return NULL;
 
   view = GTK_TEXT_VIEW (widget);
   buffer = view->buffer;
@@ -839,7 +881,7 @@ gail_text_view_get_selection (AtkText *text,
   if (*start_pos != *end_pos)
     return gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
   else
-    return 0;
+    return NULL;
 }
 
 static gboolean
@@ -1402,7 +1444,7 @@ _gail_text_view_insert_text_cb (GtkTextBuffer *buffer,
 
   gail_text_view->signal_name = "text_changed::insert";
   position = gtk_text_iter_get_offset (arg1);
-  length = arg3;
+  length = g_utf8_strlen(arg2, arg3);
   
   if (gail_text_view->length == 0)
     {
@@ -1638,6 +1680,8 @@ insert_idle_handler (gpointer data)
   GailTextView *gail_text_view;
   GtkTextBuffer *buffer;
 
+  GDK_THREADS_ENTER ();
+
   gail_text_view = GAIL_TEXT_VIEW (data);
 
   g_signal_emit_by_name (data,
@@ -1658,5 +1702,118 @@ insert_idle_handler (gpointer data)
       emit_text_caret_moved (gail_text_view, get_insert_offset (buffer));
       gail_text_view->previous_selection_bound = get_selection_bound (buffer);
     }
+
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
+
+static void       
+atk_streamable_content_interface_init    (AtkStreamableContentIface *iface)
+{
+  g_return_if_fail (iface != NULL);
+  
+  iface->get_n_mime_types = gail_streamable_content_get_n_mime_types;
+  iface->get_mime_type = gail_streamable_content_get_mime_type;
+  iface->get_stream = gail_streamable_content_get_stream;
+}
+
+static gint       gail_streamable_content_get_n_mime_types (AtkStreamableContent *streamable)
+{
+    gint n_mime_types = 0;
+
+    if (GAIL_IS_TEXT_VIEW (streamable) && GAIL_TEXT_VIEW (streamable)->textutil)
+    {
+	int i;
+	gboolean advertises_plaintext = FALSE;
+	GdkAtom *atoms =
+	     gtk_text_buffer_get_serialize_formats (
+		GAIL_TEXT_VIEW (streamable)->textutil->buffer, 
+		&n_mime_types);
+	for (i = 0; i < n_mime_types-1; ++i)
+	    if (!strcmp ("text/plain", gdk_atom_name (atoms[i])))
+		advertises_plaintext = TRUE;
+	if (!advertises_plaintext) ++n_mime_types; 
+        /* we support text/plain even if the GtkTextBuffer doesn't */
+    }
+    return n_mime_types;
+}
+
+static G_CONST_RETURN gchar*       
+gail_streamable_content_get_mime_type (AtkStreamableContent *streamable, gint i)
+{
+    if (GAIL_IS_TEXT_VIEW (streamable) && GAIL_TEXT_VIEW (streamable)->textutil)
+    {
+	gint n_mime_types = 0;
+	GdkAtom *atoms;
+	atoms = gtk_text_buffer_get_serialize_formats (
+	    GAIL_TEXT_VIEW (streamable)->textutil->buffer, 
+	    &n_mime_types);
+	if (i < n_mime_types)
+	{
+	    return gdk_atom_name (atoms [i]);
+	}
+	else if (i == n_mime_types)
+	    return "text/plain";
+    }
+    return NULL;
+}
+
+static GIOChannel*       gail_streamable_content_get_stream       (AtkStreamableContent *streamable,
+								   const gchar *mime_type)
+{
+    gint i, n_mime_types = 0;
+    GdkAtom *atoms;
+    if (!GAIL_IS_TEXT_VIEW (streamable) || !GAIL_TEXT_VIEW (streamable)->textutil)
+	return NULL;
+    atoms = gtk_text_buffer_get_serialize_formats (
+	GAIL_TEXT_VIEW (streamable)->textutil->buffer, 
+	&n_mime_types);
+    for (i = 0; i < n_mime_types; ++i) 
+    {
+	if (!strcmp ("text/plain", mime_type) ||
+	    !strcmp (gdk_atom_name (atoms[i]), mime_type)) {
+	    GtkTextBuffer *buffer;
+	    guint8 *cbuf;
+	    GError *err = NULL;
+	    gsize len, written;
+	    gchar tname[80];
+	    GtkTextIter start, end;
+	    GIOChannel *gio = NULL;
+	    int fd;
+	    buffer = GAIL_TEXT_VIEW (streamable)->textutil->buffer;
+	    gtk_text_buffer_get_iter_at_offset (buffer, &start, 0);
+	    gtk_text_buffer_get_iter_at_offset (buffer, &end, -1);
+	    if (!strcmp ("text/plain", mime_type)) 
+	    {
+		cbuf = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+		len = strlen (cbuf); 
+	    }
+	    else
+	    {
+		cbuf = gtk_text_buffer_serialize (buffer, buffer, atoms[i], &start, &end, &len);
+	    }
+	    g_snprintf (tname, 20, "streamXXXXXX");
+	    fd = g_mkstemp (tname);
+	    gio = g_io_channel_unix_new (fd);
+	    g_io_channel_set_encoding (gio, NULL, &err);
+	    if (!err) g_io_channel_write_chars (gio, cbuf, (gssize) len, &written, &err);
+	    else g_message (err->message);
+	    if (!err) g_io_channel_seek_position (gio, 0, G_SEEK_SET, &err);
+	    else g_message (err->message);
+	    if (!err) g_io_channel_flush (gio, &err);
+	    else g_message (err->message);
+	    if (err) {
+		g_message ("<error writing to stream [%s]>", tname);
+		g_free (err);
+	    }
+	    /* make sure the file is removed on unref of the giochannel */
+	    else {
+		g_unlink (tname);
+		return gio; 
+	    }
+	}
+    }
+    return NULL;
+}
+
