@@ -39,6 +39,12 @@ static AtkObject*	gail_util_get_root			(void);
 static G_CONST_RETURN gchar *gail_util_get_toolkit_name		(void);
 static G_CONST_RETURN gchar *gail_util_get_toolkit_version      (void);
 
+/* gailmisc/AtkMisc */
+static void		gail_misc_class_init			(GailMiscClass		*klass);
+
+static void gail_misc_threads_enter (AtkMisc *misc);
+static void gail_misc_threads_leave (AtkMisc *misc);
+
 /* Misc */
 
 static void		_listener_info_destroy			(gpointer		data);
@@ -344,7 +350,7 @@ gail_util_get_toolkit_version (void)
 static void
 _listener_info_destroy (gpointer data)
 {
-   free(data);
+   g_free(data);
 }
 
 static guint
@@ -477,7 +483,7 @@ window_added (AtkObject *atk_obj,
 {
   GtkWidget *widget;
 
-  g_return_if_fail (GAIL_IS_WINDOW (child));
+  if (!GAIL_IS_WINDOW (child)) return;
 
   widget = GTK_ACCESSIBLE (child)->widget;
   gail_return_if_fail (widget);
@@ -498,7 +504,7 @@ window_removed (AtkObject *atk_obj,
   GtkWidget *widget;
   GtkWindow *window;
 
-  g_return_if_fail (GAIL_IS_WINDOW (child));
+  if (!GAIL_IS_WINDOW (child)) return;
 
   widget = GTK_ACCESSIBLE (child)->widget;
   gail_return_if_fail (widget);
@@ -602,3 +608,51 @@ configure_event_watcher (GSignalInvocationHint  *hint,
       return FALSE;
     }
 }
+
+GType
+gail_misc_get_type (void)
+{
+  static GType type = 0;
+
+  if (!type)
+  {
+    static const GTypeInfo tinfo =
+    {
+      sizeof (GailMiscClass),
+      (GBaseInitFunc) NULL, /* base init */
+      (GBaseFinalizeFunc) NULL, /* base finalize */
+      (GClassInitFunc) gail_misc_class_init, /* class init */
+      (GClassFinalizeFunc) NULL, /* class finalize */
+      NULL, /* class data */
+      sizeof (GailMisc), /* instance size */
+      0, /* nb preallocs */
+      (GInstanceInitFunc) NULL, /* instance init */
+      NULL /* value table */
+    };
+
+    type = g_type_register_static (ATK_TYPE_MISC,
+                                   "GailMisc", &tinfo, 0);
+  }
+  return type;
+}
+
+static void	 
+gail_misc_class_init (GailMiscClass *klass)
+{
+  AtkMiscClass *miscclass = ATK_MISC_CLASS (klass);
+  miscclass->threads_enter =
+    gail_misc_threads_enter;
+  miscclass->threads_leave =
+    gail_misc_threads_leave;
+  atk_misc_instance = g_object_new (GAIL_TYPE_MISC, NULL);
+}
+
+static void gail_misc_threads_enter (AtkMisc *misc)
+{
+  GDK_THREADS_ENTER ();
+}
+
+static void gail_misc_threads_leave (AtkMisc *misc)
+{
+  GDK_THREADS_LEAVE ();
+}
